Learn R Programming

fusedest (version 1.3.1)

fusedest-package: Block splitting algorithms for estimation with fused penalty functions

Description

It provides functions for carrying out block splitting algorithms for fused penalty estimation.

Arguments

Details

Package: fusedest
Type: Package
Version: 1.1
Date: 2019-07-17
License: GPL-2

References

Yen, T.-J. (2019). Solving fused penalty estimation problems via block splitting algorithms.